
#include <stdio.h>
#include <string.h>

const int MAXLINE = 256;

class Process {
public:
  Process(const char* line, Process *next) {
    sscanf(line,"%*d %*d %d %d %*45c %[^\n\r]", &pid, &ppid, &name);
    child = sibling = parent = NULL;
    this->next = next;
  }
  ~Process() {
    if (next) delete next;
  }
  Process *GetNext() {
    return next;
  }
  Process *GetParent() {
    return parent;
  }
  bool IsChild(Process *child) {
    return (child->ppid == pid);
  }
  void AddChild(Process *child) {
    child->sibling = this->child;
    this->child = child;
    child->parent = this;
  }
  void Print(const char *indent = "") {
    printf("%s+-%d %s\n",indent,pid,name);
    if (child) {
      int len = strlen(indent);
      char *tmp = new char[len+3];
      memcpy(tmp,indent,len);
      tmp[len++] = (sibling) ? '|' : ' ';
      tmp[len++] = ' ';
      tmp[len] = '\0';
      child->Print(tmp);
      delete tmp;
    }
    if (sibling) sibling->Print(indent);
  }
private:
  Process *next,*child,*parent,*sibling;
  int pid,ppid;
  char name[80];
};

Process *first;

int main() {
  FILE *fpin = popen("ps -la", "r");
  if (!fpin) {
    printf("Error executing ps\n");
    return(1);
  }
  
  first = NULL;
  char line[MAXLINE];
  fgets(line, MAXLINE, fpin);
  while (fgets(line, MAXLINE, fpin)) first = new Process(line,first);
  pclose(fpin);

  Process *act;
  for (act = first; act; act=act->GetNext())
    for (Process *parent = first; parent; parent=parent->GetNext())
      if (parent->IsChild(act)) {
        parent->AddChild(act);
        break;
      }

  for (act = first; act; act=act->GetNext())
    if (!act->GetParent()) act->Print();

  delete first;

  return(0);
}
